@@ -1162,6 +1162,7 @@ class GitDeployView(GenericView):
         gddoc = self.datasvc.gitsvc.GetGitDeploy(self.context.application, self.context.name)
         #re-reference embedded
         gddoc['location']['gitrepo'] = gddoc['location']['gitrepo']['name']
+        gddoc['created_datetime'] = self.get_created_datetime_text()
         return {k: gddoc[k] for k in gddoc if k[0] != '_'}
 
     def check_servers_list(self, body):
@@ -1602,6 +1603,15 @@ class UserView(GenericView):
         logging.debug("allowed_apps: {}".format(authsvc.get_allowed_apps()))
         return False, self.Error(403, "bad token")
 
+    def DELETE(self):
+        ok, err = self.check_password()
+        if not ok:
+            return err
+        if not self.global_write:
+            return self.Error(403, "only admins can delete users")
+        self.datasvc.usersvc.DeleteUser(self.context.username)
+        return self.status_ok({"user_deleted": {"username": self.context.username}})
+
     def GET(self):
         # another ugly. we want to support both password auth and valid tokens, but only tokens from the same user or
         #  with the '_global' permission.
@@ -1651,7 +1661,6 @@ class UserPermissionsView(GenericView):
         return auth.UserPermissions(self.datasvc.usersvc, None, datasvc=self.datasvc).get_allowed_actions(self
                                                                                                           .context
                                                                                                           .username)
-
     def compute_server_perms(self):
         return auth.UserPermissions(self.datasvc.usersvc, None, datasvc=self.datasvc).get_allowed_servers(self
                                                                                                           .context
